import Foundation
import UIKit

class StatusBarColorizer: NSObject
{
    private var active = false;
    private var view: UIView?
    private static let instance = StatusBarColorizer()

    override init()
    {
        super.init()
        StatusBarColorizer.delegate().addObserver(self,
                                                  forKeyPath: "window",
                                                  options: [.new, .initial],
                                                  context: nil)

        NotificationCenter.default.addObserver(self,
                                               selector: #selector(didRotate),
                                               name: NSNotification.Name.UIDeviceOrientationDidChange,
                                               object: nil)
    }

    deinit
    {
        StatusBarColorizer.delegate().removeObserver(self,
                                                     forKeyPath: "window")

        NotificationCenter.default.removeObserver(self)
    }

    override func observeValue(forKeyPath keyPath: String?,
                               of object: Any?,
                               change: [NSKeyValueChangeKey : Any]?,
                               context: UnsafeMutableRawPointer?)
    {
        if(active)
        {
            StatusBarColorizer.addBarTo(window: StatusBarColorizer.window())
        }
    }

    @objc func didRotate()
    {
        StatusBarColorizer.bar().frame = CGRect(x: UIApplication.shared.statusBarFrame.origin.x,
                                                y: UIApplication.shared.statusBarFrame.origin.y,
                                                width: UIApplication.shared.statusBarFrame.size.width,
                                                height: UIApplication.shared.statusBarFrame.size.height)
    }

    static func show()
    {
        if(!instance.active)
        {
            instance.active = true
            addBarTo(window: window())
        }
    }

    static func hide()
    {
        if(instance.active)
        {
            instance.active = false
            bar().removeFromSuperview()
        }
    }

    static func color(_ color: UIColor)
    {
        bar().backgroundColor = color;
    }

    private static func addBarTo(window: UIWindow?)
    {
        bar().removeFromSuperview()
        bar().frame = CGRect(x: UIApplication.shared.statusBarFrame.origin.x,
                             y: UIApplication.shared.statusBarFrame.origin.y,
                             width: UIApplication.shared.statusBarFrame.size.width,
                             height: UIApplication.shared.statusBarFrame.size.height)
        window?.addSubview(bar());
    }

    private static func bar() -> UIView
    {
        if(instance.view == nil)
        {
            instance.view = UIView(frame: CGRect(x: UIApplication.shared.statusBarFrame.origin.x,
                                                 y: UIApplication.shared.statusBarFrame.origin.y,
                                                 width: UIApplication.shared.statusBarFrame.size.width,
                                                 height: UIApplication.shared.statusBarFrame.size.height))
            instance.view?.layer.zPosition = 1000;
            instance.view?.backgroundColor = UIColor.orange
            instance.view?.autoresizingMask = [.flexibleWidth]
        }
        return instance.view!
    }

    private static func window() -> UIWindow?
    {
        return (UIApplication.shared.delegate?.window)!
    }

    private static func delegate() -> NSObject!
    {
        return (UIApplication.shared.delegate as! NSObject)
    }
}
