import Flutter
import UIKit

public class SwiftStatusbarPlugin: NSObject, FlutterPlugin
{
    public static func register(with registrar: FlutterPluginRegistrar)
    {
        let channel = FlutterMethodChannel(name: "statusbar", binaryMessenger: registrar.messenger())
        let instance = SwiftStatusbarPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult)
    {
        if(call.method == "show")
        {
            StatusBarColorizer.show()
            result("done")
        }
        else if(call.method == "hide")
        {
            StatusBarColorizer.hide()
            result("done")
        }
        else if(call.method == "color")
        {
            let color = colorFromString((call.arguments as? Dictionary<String, Any>)?["hex"] as? String)
            StatusBarColorizer.color(color)
            result("done")
        }
        else
        {
            result(FlutterMethodNotImplemented)
        }
    }
    
    private func colorFromString(_ hex: String?) -> UIColor
    {
        var cString:String = (hex ?? "").trimmingCharacters(in: .whitespacesAndNewlines).uppercased()
        
        if (cString.hasPrefix("#")) {
            cString.remove(at: cString.startIndex)
        }
        
        if ((cString.count) != 6) {
            return UIColor.gray
        }
        
        var rgbValue:UInt32 = 0
        Scanner(string: cString).scanHexInt32(&rgbValue)
        
        return UIColor(
            red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
            alpha: CGFloat(1.0)
        )
    }
}
